# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-11-20 09:16:53 +0900 (Thu, 20 Nov 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/file'
require 'vfs/scheme/file/common'

module VFS;  module SchemeHandler; module FILE;
  class File < VFS::File
    include VFS::SchemeHandler::FILE::Common
    extend  VFS::SchemeHandler::FILE::MethodGenerator

    relay_realpath_to_file_builtin :size?,
      :chardev?, :zero?, :writable_real?,
      :atime, :pipe?, :exists?, :symlink, :sticky?, :file?, :stat,
      :blockdev?, :writable?, :ftype, :foreach,
      :grpowned?, :exist?, :link, :setgid?, :executable_real?,
      :delete, :ctime, :socket?, :readable_real?, :truncate, :unlink,
      :lstat, :owned?, :directory?, :setuid?, :executable?,
      :expand_path, :mtime, :symlink?, :size, :readable?, :readlink

    class_eval [:uid, :dev, :rdev_major, :nlink, :blocks, :rdev,
                :mode, :blksize, :gid, :ino, :rdev_minor].map {|m| %{
        def #{m}(*args)
          FileBuiltIn::Stat.new(local_path).#{m}(*args)
        end
      } }.join

    undef :buf, :open_for_read, :open_for_write

    def open(*arg, &block)
      FileBuiltIn.open(local_path, *arg, &block)
    end

    def rename_internal(dest)
      begin
        FileBuiltIn.rename(local_path, dest)
        VFS::File.new(dest)
      rescue
        rename_external(dest)
      end
    end

    def mtime=(new_mtime)
      FileBuiltIn.utime(atime, new_mtime, local_path)
      new_mtime
    end

    def atime=(new_atime)
      FileBuiltIn.utime(new_atime, mtime, local_path)
      new_atime
    end

    def mode=(new_mode)
      FileBuiltIn.chmod(new_mode, local_path)
      new_mode
    end

    def lmode=(new_mode)
      FileBuiltIn.lchmod(new_mode, local_path)
      new_mode
    end

    def uid=(new_id)
      FileBuiltIn.chown(new_id, gid, local_path)
      new_id
    end

    def gid=(new_gid)
      FileBuiltIn.chown(uid, new_id, local_path)
      new_gid
    end

    def luid=(new_id)
      FileBuiltIn.lchown(new_id, gid, local_path)
      new_id
    end

    def lgid=(new_gid)
      FileBuiltIn.lchown(uid, new_id, local_path)
      new_gid
    end

  end # VFS::SchemeHandler::FILE::File
end; end; end

if __FILE__ == $0
  uri = URI.parse("file:///home/sugi/.bashrc")
  vfs = VFS::SchemeHandler::FILE::File.new(uri)
  p vfs
  p vfs.join(%w(a b c))
  p VFS::join(%w(a b c))
  p VFS::scheme(uri)
  p vfs.scheme
  p vfs.mtime
#  p VFS::SchemeHandler::FILE::File.mtime(f)
  p vfs.basename
  p vfs.dirname
  p vfs.scheme
  io = vfs.open
  puts io.read(20)
  io.close
  vfs.open { |io| puts io.read(20) }
#  io = VFS::SchemeHandler::FILE::File.open(f)
#  puts io.read(20)
#  io.close
#  VFS::SchemeHandler::FILE::File.open(f){ |io| puts io.read(20) }
#  VFS::SchemeHandler::FILE::File.new("/tmp/moge") << ["mogemoge", Time.new]
end
