#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-11-12 18:00:17 +0900 (Wed, 12 Nov 2003) $
#   $LastChangedBy: sugi $
#

require 'net/ftp'

module VFS;  module SchemeHandler;  module FTP;
  class ConnectPool
    @@pool = Hash.new
    CALLER = 0
    CONN   = 1
    class << self
      def connect(caller_id, uri)
        conn_id = "#{uri.user}@#{uri.host}:#{uri.port}"
        if ! @@pool.has_key?(conn_id)
          @@pool[conn_id] = Array.new
          @@pool[conn_id][CONN] = Net::FTP.new
          log("Opening FTP connection to #{conn_id}.", Log::INFO)
          @@pool[conn_id][CALLER] = [caller_id]
        elsif @@pool[conn_id][CALLER].member? caller_id
          @@pool[conn_id][CALLER] << caller_id
        end

        if @@pool[conn_id][CONN].closed?
          @@pool[conn_id][CONN].connect(uri.host, uri.port)
          @@pool[conn_id][CONN].login(
            uri.user ? URI.unescape(uri.user) : "anonymous",
            uri.password ? URI.unescape(uri.password) : nil)
        end

        if block_given?
          yield(@@pool[conn_id][CONN])
        else
          @@pool[conn_id][CONN]
        end

      end

      def request(caller_id, uri)
        c = connect(caller_id, uri)
        if block_given?
          yield(c)
        else
          c
        end
      end

      def release(caller_id)
        @@pool.each { |conn_id, pool|
          pool[CALLER].delete(caller_id)
          if pool[CALLER].empty?
            unless pool[CONN].closed?
              log("Closing connection for #{conn_id}.", Log::INFO)
              pool[CONN].quit
            end
            @@pool.delete conn_id
          end
        }
      end

      def log(msg, level)
        VFS::Log.log("VFS(HTTP): #{msg}", level)
      end
    end
  end

  module Common

    def init_common
      @conn   = Net::FTP.new
      ObjectSpace.define_finalizer(self,
                                   Proc.new { |id| 
          Proc.new { ConnectPool.release(id) }
      }.call(self.__id__))
    end

    def close_socket
      ConnectPool.connect(self.__id__, uri).quit
    end

    def ftp_request(&block)
      ConnectPool.request(self.__id__, uri, &block)
    end

  end
end; end; end
